
/*-----( Import needed libraries )-----*/
#include <SM.h>//state machine library
#include <SPI.h>
#include <Mcp23s17.h>

#define circuit1_SLAVE_SELECT_PIN  5 //arduino   <->   SPI Slave Select   -> CS  (Pin 11 on MCP23S17 DIP)
#define circuit2_SLAVE_SELECT_PIN  6 //arduino   <->   SPI Slave Select    -> CS  (Pin 11 on MCP23S17 DIP)

MCP23S17 circuit1 = MCP23S17( circuit1_SLAVE_SELECT_PIN, 0x01 );//addresse 1
MCP23S17 circuit2 = MCP23S17( circuit2_SLAVE_SELECT_PIN, 0x02 );//addresse 2

int etat=0;// used for RE (RISING EDGE) and FE (FALLING EDGE)
uint16_t pinstate1;//to read the word input/output circuit1
uint16_t pinstate2;//to read the word input/output circReceive uit2
int memo;
uint16_t LEDWORD;
float VALdac1=0;
float VALdac2=0;
word outputValue1 = 0; // a word is a 16-bit number
byte data1 = 0; // and a byte is an 8-bit number

/*-----( Declare objects )-----*/
SM Master(Master1);             //the machines and their first state to run
SM FirstGR7(STATE0);
SM SecondGR7(STATE10);


void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(115200);
  pinMode(2,INPUT);
  pinMode(3,INPUT);
  pinMode(8,OUTPUT);
  pinMode(9,OUTPUT);
  pinMode(10,OUTPUT);
  pinMode(circuit1_SLAVE_SELECT_PIN,OUTPUT);
  pinMode(circuit2_SLAVE_SELECT_PIN,OUTPUT);


  //circuit1.pinMode(0,INPUT);//INTERNAL PULLUP NOT necessary here
  //  unless both input would be activated,WHEN INPUT AND OUTPUT ON THE SAME CIRCUIT PORT
  //circuit1.pinMode(1,INPUT);
  circuit1.pinMode(2,OUTPUT);
  circuit1.pinMode(3,OUTPUT);
  circuit1.digitalWrite(2,LOW);
  circuit1.digitalWrite(3,LOW);

  //circuit2.pinMode(0,INPUT);
  //circuit2.pinMode(1,INPUT);
  //circuit2.pinMode(2,INPUT);
  circuit2.pinMode(3,OUTPUT);
  circuit2.pinMode(4,OUTPUT);
  circuit2.pinMode(5,OUTPUT);
  circuit2.pinMode(6,OUTPUT);
  circuit2.digitalWrite(3,LOW);
  circuit2.digitalWrite(4,LOW);
  circuit2.digitalWrite(5,LOW);
  circuit2.digitalWrite(6,LOW);

  //circuit2.port(0x0000);//toutes les sorties à 0
  SPI.begin(); // wake up the SPI bus.
  SPI.setBitOrder(MSBFIRST);

}//--(end setup )---


void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  pinstate1 = circuit1.port();//read input in a word 
  pinstate2 = circuit2.port();
  //Serial.println(pinstate1,BIN);//read the ports state to be sure of the result of push buttons
  //Serial.println(pinstate1,BIN);

  EXEC(Master);//launch Master state machine


  //emergency stop/run/stop machine
  if(RE(bitRead(pinstate1,1),etat)==1){ //@1A1 Stops the Master machine, (emergency stop), fourth button on schematic
    //problems with circuit1.digitalRead(), bitRead() suitable
    //@1A2=@1A3=@2A3=@2A4=@2A5=@2A6=OFF
    circuit1.digitalWrite(2,LOW);
    circuit1.digitalWrite(3,LOW);
    circuit2.digitalWrite(3,LOW);
    circuit2.digitalWrite(4,LOW);
    circuit2.digitalWrite(5,LOW);
    circuit2.digitalWrite(6,LOW);
    //C1ANA=C2ANA=0V
    outputValue1 = 0;
    digitalWrite(9, LOW);
    data1 = highByte(outputValue1);
    data1 = 0b00001111 & data1;
    data1= 0b00110000 | data1;
    SPI.transfer(data1);
    data1 = lowByte(outputValue1);
    SPI.transfer(data1);
    digitalWrite(9, HIGH);
    delay(50);

    outputValue1 = 0;
    digitalWrite(10, LOW);
    data1 = highByte(outputValue1);
    data1 = 0b00001111 & data1;
    data1= 0b00110000 | data1;
    SPI.transfer(data1);
    data1 = lowByte(outputValue1);
    SPI.transfer(data1);
    digitalWrite(10, HIGH);
    delay(50);

    Master.Finish();//stop the Master state machine
  }

  if ((RE(bitRead(pinstate1,0),etat)==1)&& Master.Finished //Rising edge on @1A0 RESTARTS the Master machine
  )  
  {
    EXEC(Master);            //Launch the Master machine
    FirstGR7.Set(STATE0); //activate the first state of FirstGR7 because i used a push button
    //in it to begin the SD sequence
    SecondGR7.Set(STATE10);
    Master.Set(Master2);     //second state activation, necessary
  }

}//--(end main loop )---

/*-----( Declare User-written Functions )-----*/

//---( STATE MACHINE: Master )------------------master launch

State Master1() //waiting for start push button
{


  //Serial.println("Master launched");
  if (RE(bitRead(pinstate1,0),etat)==1) { //Rising edge on @1A1 starts the Master2 which lauches the other machines   
    Master.Set(Master2);
  }
}

State Master2() //launch the slave machines
{
  EXEC(FirstGR7);
  EXEC(SecondGR7);
}

//---( STATE MACHINE: FirstGR7 )-----------------------------------------------------
State STATE0()    // Initial Delay from Sync
{
  circuit2.digitalWrite(3,LOW);
  circuit2.digitalWrite(4,LOW);
  circuit2.digitalWrite(5,LOW);
  circuit2.digitalWrite(6,LOW);

  if (RE(bitRead(pinstate2,0),etat)==1) { //Rising edge on @1A1 starts the Master2 which lauches the other machines   
    FirstGR7.Set(STATE1) ; 
  }

}

State STATE1()    // Initial Delay from Sync
{
  circuit2.digitalWrite(3,HIGH);
  circuit2.digitalWrite(4,LOW);
  circuit2.digitalWrite(5,LOW);
  circuit2.digitalWrite(6,LOW);
  if (RE(bitRead(pinstate2,0),etat)==1) FirstGR7.Set(STATE2) ; 
}

State STATE2()
{
  circuit2.digitalWrite(3,LOW);
  circuit2.digitalWrite(4,HIGH);
  circuit2.digitalWrite(5,LOW);
  circuit2.digitalWrite(6,LOW);
  if (RE(bitRead(pinstate2,0),etat)==1) FirstGR7.Set(STATE3) ;
}

State STATE3()
{
  circuit2.digitalWrite(3,LOW);
  circuit2.digitalWrite(4,LOW);
  circuit2.digitalWrite(5,HIGH);
  circuit2.digitalWrite(6,LOW);
  if (RE(bitRead(pinstate2,0),etat)==1) FirstGR7.Set(STATE4) ;
}

State STATE4()
{
  circuit2.digitalWrite(3,LOW);
  circuit2.digitalWrite(4,LOW);
  circuit2.digitalWrite(5,LOW);
  circuit2.digitalWrite(6,HIGH);
  if (RE(bitRead(pinstate2,0),etat)==1) 
    FirstGR7.Set(STATE5) ;
}

State STATE5()
{
  circuit2.digitalWrite(3,HIGH);
  circuit2.digitalWrite(4,HIGH);
  circuit2.digitalWrite(5,HIGH);
  circuit2.digitalWrite(6,HIGH);
  LEDWORD=1;
  if (RE(bitRead(pinstate1,8),etat)==1) FirstGR7.Set(STATE6);
  if (RE(bitRead(pinstate2,0),etat)==1) FirstGR7.Set(STATE1);
  if (RE(bitRead(pinstate2,1),etat)==1) FirstGR7.Set(STATE0);

}

State STATE6()
{
  if (LEDWORD>8) {
    LEDWORD=1;
  }
  circuit2.digitalWrite(3,bitRead(LEDWORD,0));
  circuit2.digitalWrite(4,bitRead(LEDWORD,1));
  circuit2.digitalWrite(5,bitRead(LEDWORD,2));
  circuit2.digitalWrite(6,bitRead(LEDWORD,3));
  if (FirstGR7.Timeout(500)){
    LEDWORD=LEDWORD*2;
    FirstGR7.Set(STATE6) ;
  }
  if (RE(bitRead(pinstate2,1),etat)==1) FirstGR7.Set(STATE0);
}


//---( STATE MACHINE: SecondGR7 )---------------------------------------------------
State STATE10()    // Initial Delay from Sync
{
  circuit1.digitalWrite(2,HIGH);
  VALdac1=0;
  VALdac2=0;
  if (SecondGR7.Timeout(2000)) {

    SecondGR7.Set(STATE11) ; 
  } 
}

State STATE11()    // Initial Delay from Sync
{
  circuit1.digitalWrite(2,LOW);

  outputValue1 = VALdac1;
  digitalWrite(9, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1= 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(9, HIGH);

  outputValue1 = VALdac2;
  digitalWrite(10, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1= 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(10, HIGH);

  if (SecondGR7.Timeout(50)){
    VALdac1=VALdac1+100;
    VALdac2=VALdac2+100;
    SecondGR7.Set(STATE11);
  }
  if (VALdac1>4095 || VALdac2>4095) {
    VALdac1=4095;
    VALdac2=4095;
    SecondGR7.Set(STATE12);
  }
}

State STATE12()
{

  circuit1.digitalWrite(2,HIGH);

  outputValue1 = VALdac1;
  digitalWrite(9, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1= 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(9, HIGH);

  outputValue1 = VALdac2;
  digitalWrite(10, LOW);
  data1 = highByte(outputValue1);
  data1 = 0b00001111 & data1;
  data1= 0b00110000 | data1;
  SPI.transfer(data1);
  data1 = lowByte(outputValue1);
  SPI.transfer(data1);
  digitalWrite(10, HIGH);

  if (SecondGR7.Timeout(50)){
    VALdac1=VALdac1-100;
    VALdac2=VALdac2-100;
    SecondGR7.Set(STATE12);
  }
  if (VALdac1<1 || VALdac2<1) {
    VALdac1=0;
    VALdac2=0;
    SecondGR7.Set(STATE13);
  }
}


State STATE13()
{
  circuit1.digitalWrite(2,LOW);
  if (SecondGR7.Timeout(500))  SecondGR7.Set(STATE10) ;

}



//*********( THE END )***********




























































